@@ -6,12 +6,16 @@ from encourage.metrics.classic import (
     F1,
     GLEU,
     ROUGE,
+    Accuracy,
     BERTScore,
     ContextLength,
     ExactMatch,
+    F1Classification,
     GeneratedAnswerLength,
     HitRateAtK,
     MeanReciprocalRank,
+    Precision,
+    Recall,
     RecallAtK,
     ReferenceAnswerLength,
     ROUGEDetailed,
@@ -28,29 +32,33 @@ from encourage.metrics.registry import (
 )
 
 __all__ = [
+    "Accuracy",
     "AnswerFaithfulness",
     "AnswerRelevance",
     "AnswerSimilarity",
-    "BLEU",
-    "F1",
-    "GLEU",
-    "ROUGE",
-    "ROUGEDetailed",
     "BERTScore",
+    "BLEU",
     "ContextLength",
-    "ExactMatch",
-    "GeneratedAnswerLength",
-    "MeanReciprocalRank",
-    "ReferenceAnswerLength",
     "ContextPrecision",
     "ContextRecall",
+    "ExactMatch",
+    "F1Classification",
+    "F1",
+    "GeneratedAnswerLength",
+    "get_metric_from_registry",
+    "GLEU",
+    "HitRateAtK",
+    "MeanReciprocalRank",
+    "METRIC_REGISTRY",
     "Metric",
     "MetricOutput",
     "NonAnswerCritic",
     "NumberMatch",
+    "Precision",
+    "Recall",
     "RecallAtK",
-    "HitRateAtK",
-    "METRIC_REGISTRY",
-    "get_metric_from_registry",
+    "ReferenceAnswerLength",
     "register_metric",
+    "ROUGE",
+    "ROUGEDetailed",
 ]
