@@ -1,5 +1,7 @@
 """Classic metrics for evaluating RAG."""
 
+import re
+import uuid
 from functools import cached_property
 from typing import Any, Union
 
@@ -9,6 +11,7 @@ import numpy as np
 from encourage.llm.response_wrapper import ResponseWrapper
 from encourage.metrics.metric import Metric, MetricOutput
 from encourage.metrics.registry import register_metric
+from encourage.prompts.context import Document
 
 
 @register_metric("GeneratedAnswerLength")
@@ -95,8 +98,8 @@ class BLEU(Metric):
     def __call__(self, responses: ResponseWrapper) -> MetricOutput:
         """Calls the metric calculation."""
         output = self.metric.compute(
-            predictions=[r.response for r in responses],
-            references=[[r.meta_data["reference_answer"]] for r in responses],
+            predictions=[str(r.response) for r in responses],
+            references=[[str(r.meta_data["reference_answer"])] for r in responses],
             max_order=self.n_grams,
         )
 
@@ -123,8 +126,8 @@ class GLEU(Metric):
     def __call__(self, responses: ResponseWrapper) -> MetricOutput:
         """Calls the metric calculation."""
         output = self.metric.compute(
-            predictions=[r.response for r in responses],
-            references=[r.meta_data["reference_answer"] for r in responses],
+            predictions=[str(r.response) for r in responses],
+            references=[str(r.meta_data["reference_answer"]) for r in responses],
         )
         if output is None or "google_bleu" not in output:
             return MetricOutput(score=0.0, raw=[])
@@ -158,9 +161,9 @@ class ROUGE(Metric):
         self.validate_nested_keys(responses)
 
         output = self.metric.compute(
-            predictions=[r.response for r in responses if "reference_answer" in r.meta_data],
+            predictions=[str(r.response) for r in responses if "reference_answer" in r.meta_data],
             references=[
-                r.meta_data["reference_answer"]
+                str(r.meta_data["reference_answer"])
                 for r in responses
                 if "reference_answer" in r.meta_data
             ],
@@ -200,8 +203,8 @@ class ROUGEDetailed(Metric):
         recall = []
         f1 = []
         for ref, pred in zip(
-            [r.meta_data["reference_answer"] for r in responses],
-            [r.response for r in responses],
+            [str(r.meta_data["reference_answer"]) for r in responses],
+            [str(r.response) for r in responses],
         ):
             score = self.scorer.score(ref or "", pred)
             precision.append(score[self.rouge_type].precision)
@@ -237,8 +240,8 @@ class BERTScore(Metric):
         """Calls the metric calculation."""
         self.validate_nested_keys(responses)
         result = self.metric.compute(
-            predictions=[r.response for r in responses],
-            references=[r.meta_data["reference_answer"] for r in responses],
+            predictions=[str(r.response) for r in responses],
+            references=[str(r.meta_data["reference_answer"]) for r in responses],
             **self.metric_args,
         )
         if result is None or "f1" not in result:
@@ -256,8 +259,8 @@ class F1(Metric):
 
     def __init__(self) -> None:
         super().__init__(
-            name="f1",
-            description="F1 score for the generated answers",
+            name="f1squad",
+            description="Squad score for the generated answers",
             required_meta_data=["reference_answer"],
         )
 
@@ -265,7 +268,7 @@ class F1(Metric):
 
     @cached_property
     def load_metric(self) -> Any:
-        """Loads the F1 metric."""
+        """Loads the F1 SQuAD v2 metric."""
         from evaluate import load
 
         return load("squad_v2")
@@ -289,7 +292,7 @@ class F1(Metric):
             formatted_references.append(
                 {
                     "id": str(i),
-                    "answers": [{"text": r.meta_data["reference_answer"], "answer_start": 0}],
+                    "answers": [{"text": str(r.meta_data["reference_answer"]), "answer_start": 0}],
                 }
             )
 
@@ -306,6 +309,215 @@ class F1(Metric):
         )
 
 
+@register_metric("F1Classification")
+class F1Classification(Metric):
+    """Computes the F1 score for the generated answers."""
+
+    def __init__(self, average="binary", pos_label="True") -> None:
+        super().__init__(
+            name="f1",
+            description="F1 score for the generated answers",
+            required_meta_data=["reference_answer"],
+        )
+        self.average = average
+        self.metric = self.load_metric
+        self.pos_label = pos_label
+
+    @cached_property
+    def load_metric(self) -> Any:
+        """Loads the F1 metric."""
+        from evaluate import load
+
+        return load("f1")
+
+    def __call__(self, responses: ResponseWrapper) -> MetricOutput:
+        """Calls the metric calculation."""
+        self.validate_nested_keys(responses)
+        # Initialize empty lists for formatted predictions and references
+        formatted_predictions = []
+        formatted_references = []
+
+        # Iterate over predictions and references
+        for r in responses:
+            formatted_predictions.append(1 if str(r.response) == self.pos_label else 0)
+            formatted_references.append(
+                1 if str(r.meta_data["reference_answer"]) == self.pos_label else 0
+            )
+
+        # Call the compute function with formatted data
+        output = self.metric.compute(
+            predictions=formatted_predictions,
+            references=formatted_references,
+            average=self.average,
+            pos_label=1,
+        )
+        if output is None or "f1" not in output:
+            return MetricOutput(score=0.0, raw=[])
+        return MetricOutput(
+            score=output["f1"],
+            raw=output["f1"],
+            misc={
+                "average": self.average,
+                "pos_label": self.pos_label,
+                "predictions": str(formatted_predictions),
+                "references": str(formatted_references),
+            },
+        )
+
+
+@register_metric("Precision")
+class Precision(Metric):
+    """Computes the Precision score for the generated answers."""
+
+    def __init__(self, average="binary") -> None:
+        super().__init__(
+            name="precision",
+            description="Precision score for the generated answers",
+            required_meta_data=["reference_answer"],
+        )
+        self.average = average
+        self.metric = self.load_metric
+
+    @cached_property
+    def load_metric(self) -> Any:
+        """Loads the Precision metric."""
+        from evaluate import load
+
+        return load("precision")
+
+    def __call__(self, responses: ResponseWrapper) -> MetricOutput:
+        """Calls the metric calculation."""
+        self.validate_nested_keys(responses)
+        # Initialize empty lists for formatted predictions and references
+        formatted_predictions = []
+        formatted_references = []
+
+        # Use zip to iterate over predictions and references
+        for r in responses:
+            formatted_predictions.append(1 if str(r.response).lower() == "true" else 0)
+            formatted_references.append(
+                1 if str(r.meta_data["reference_answer"]).lower() == "true" else 0
+            )
+
+        # Call the compute function with formatted data
+        output = self.metric.compute(
+            predictions=formatted_predictions, references=formatted_references, average=self.average
+        )
+        if output is None or "precision" not in output:
+            return MetricOutput(score=0.0, raw=[])
+        return MetricOutput(
+            score=output["precision"],
+            raw=output["precision"],
+            misc={
+                "average": self.average,
+                "predictions": str(formatted_predictions),
+                "references": str(formatted_references),
+            },
+        )
+
+
+@register_metric("Recall")
+class Recall(Metric):
+    """Computes the Recall score for the generated answers."""
+
+    def __init__(self, average="binary") -> None:
+        super().__init__(
+            name="recall",
+            description="Recall score for the generated answers",
+            required_meta_data=["reference_answer"],
+        )
+        self.average = average
+        self.metric = self.load_metric
+
+    @cached_property
+    def load_metric(self) -> Any:
+        """Loads the Recall metric."""
+        from evaluate import load
+
+        return load("recall")
+
+    def __call__(self, responses: ResponseWrapper) -> MetricOutput:
+        """Calls the metric calculation."""
+        self.validate_nested_keys(responses)
+        # Initialize empty lists for formatted predictions and references
+        formatted_predictions = []
+        formatted_references = []
+
+        # Use zip to iterate over predictions and references
+        for r in responses:
+            formatted_predictions.append(1 if str(r.response).lower() == "true" else 0)
+            formatted_references.append(
+                1 if str(r.meta_data["reference_answer"]).lower() == "true" else 0
+            )
+
+        # Call the compute function with formatted data
+        output = self.metric.compute(
+            predictions=formatted_predictions, references=formatted_references, average=self.average
+        )
+        if output is None or "recall" not in output:
+            return MetricOutput(score=0.0, raw=[])
+        return MetricOutput(
+            score=output["recall"],
+            raw=output["recall"],
+            misc={
+                "average": self.average,
+                "predictions": str(formatted_predictions),
+                "references": str(formatted_references),
+            },
+        )
+
+
+@register_metric("Accuracy")
+class Accuracy(Metric):
+    """Computes the Accuracy score."""
+
+    def __init__(self, average="binary") -> None:
+        super().__init__(
+            name="accuracy",
+            description="Accuracy for the generated answers",
+            required_meta_data=["reference_answer"],
+        )
+        self.average = average
+        self.metric = self.load_metric
+
+    @cached_property
+    def load_metric(self) -> Any:
+        """Loads the Accuracy metric."""
+        from evaluate import load
+
+        return load("accuracy")
+
+    def __call__(self, responses: ResponseWrapper) -> MetricOutput:
+        """Calls the metric calculation."""
+        self.validate_nested_keys(responses)
+        # Initialize empty lists for formatted predictions and references
+        formatted_predictions = []
+        formatted_references = []
+
+        # Use zip to iterate over predictions and references
+        for r in responses:
+            formatted_predictions.append(1 if str(r.response).lower() == "true" else 0)
+            formatted_references.append(
+                1 if str(r.meta_data["reference_answer"]).lower() == "true" else 0
+            )
+
+        # Call the compute function with formatted data
+        output = self.metric.compute(
+            predictions=formatted_predictions,
+            references=formatted_references,
+        )
+        if output is None or "accuracy" not in output:
+            return MetricOutput(score=0.0, raw=[])
+        return MetricOutput(
+            score=output["accuracy"],
+            raw=output["accuracy"],
+            misc={
+                "predictions": str(formatted_predictions),
+                "references": str(formatted_references),
+            },
+        )
+
+
 @register_metric("DROP_F1")
 class DropF1(Metric):
     """DROP-style F1 score with strict number match and multi-span alignment."""
@@ -323,7 +535,6 @@ class DropF1(Metric):
 
         def normalize_answer(s: str) -> str:
             """Lower text and remove punctuation, articles, and extra whitespace."""
-            import re
             import string
 
             def remove_articles(text: str) -> str:
@@ -361,8 +572,8 @@ class DropF1(Metric):
 
         scores: list[float] = []
         for r in responses:
-            prediction: str = r.response
-            references: Union[str, list[str]] = r.meta_data["reference_answer"] or []
+            prediction: str = str(r.response)
+            references: Union[str, list[str]] = str(r.meta_data["reference_answer"]) or []
             if isinstance(references, str):
                 references = [references]
             f1_scores = [compute_f1(ref, prediction) for ref in references]
@@ -402,17 +613,16 @@ class ExactMatch(Metric):
             formatted_predictions.append(
                 {
                     "id": str(i),
-                    "prediction_text": r.response,
+                    "prediction_text": str(r.response),
                     "no_answer_probability": 0.0,
                 }
             )
             formatted_references.append(
                 {
                     "id": str(i),
-                    "answers": [{"text": r.meta_data["reference_answer"], "answer_start": 0}],
+                    "answers": [{"text": str(r.meta_data["reference_answer"]), "answer_start": 0}],
                 }
             )
-
         # Call the compute function with formatted data
         output = self.metric.compute(
             predictions=formatted_predictions,
@@ -435,7 +645,15 @@ class RetrievalMetric(Metric):
         qrels, run = {}, {}
         for response in responses:
             query_id = response.request_id
-            relevant = {str(response.meta_data["reference_document"].id): 1}  # type: ignore
+            ref_doc = response.meta_data["reference_document"]
+            if ref_doc is None:
+                ref_doc = Document(id=uuid.UUID(int=0), content="")
+            if isinstance(ref_doc, list):
+                relevant = {str(doc.id): 1 for doc in ref_doc}  # type: ignore
+            elif isinstance(ref_doc, Document):
+                relevant = {str(ref_doc.id): 1}
+            else:
+                raise ValueError("reference_document must be a Document or list of Documents")
             retrieved = {
                 str(document.id): document.score for document in response.context.documents
             }
@@ -465,6 +683,48 @@ class MeanReciprocalRank(RetrievalMetric):
         return MetricOutput(score=float(np.mean(scores)), raw=scores)
 
 
+@register_metric("MeanAveragePrecision")
+class MeanAveragePrecision(RetrievalMetric):
+    """Computes the Mean Average Precision (MAP) for the responses."""
+
+    def __init__(self) -> None:
+        super().__init__(
+            name="map",
+            description="Mean Average Precision (MAP) for the responses",
+            required_meta_data=["reference_document"],
+            required_documents=True,
+        )
+
+    def __call__(self, responses: ResponseWrapper) -> MetricOutput:
+        """Calls the metric calculation."""
+        self.validate_nested_keys(responses)
+        qrels, run = self.responses_to_trec(responses)
+        map_metric = ir_measures.AP()
+        scores = [score.value for score in map_metric.iter_calc(qrels, run)]
+        return MetricOutput(score=float(np.mean(scores)), raw=scores)
+
+
+@register_metric("NDCG")
+class NDCG(RetrievalMetric):
+    """Computes the Normalized Discounted Cumulative Gain (NDCG) for the responses."""
+
+    def __init__(self) -> None:
+        super().__init__(
+            name="ndcg",
+            description="Normalized Discounted Cumulative Gain (NDCG) for the responses",
+            required_meta_data=["reference_document"],
+            required_documents=True,
+        )
+
+    def __call__(self, responses: ResponseWrapper) -> MetricOutput:
+        """Calls the metric calculation."""
+        self.validate_nested_keys(responses)
+        qrels, run = self.responses_to_trec(responses)
+        ndcg_metric = ir_measures.NDCG()
+        scores = [score.value for score in ndcg_metric.iter_calc(qrels, run)]
+        return MetricOutput(score=float(np.mean(scores)), raw=scores)
+
+
 @register_metric("RecallAtK")
 class RecallAtK(RetrievalMetric):
     """Computes Recall@k for the responses."""
